/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */

/**
 * Dialog to display a user's user-token details.
 *
 * @class Nexus.usertoken.UserTokenWindow
 * @extends Ext.Window
 * @namespace Nexus.usertoken
 */
NX.define('Nexus.usertoken.UserTokenWindow', {
  extend: 'Ext.Window',

  requires: [
    'Nexus.usertoken.Icons'
  ],
  requirejs: ['Nexus/usertoken'],

  /**
   * @cfg {String} nameCode   The user-token name-code.
   * @cfg {String} passCode   The user-token pass-code.
   */
  constructor: function (config) {
    var self = this,
        icons = Nexus.usertoken.Icons,
        exampleXml;

    Ext.applyIf(config, {
      message: 'You have requested an operation which requires validation of your credentials.'
    });

    exampleXml = '<server>\n' +
        '  <id>${server}</id>\n' +
        '  <username>' + config.nameCode + '</username>\n' +
        '  <password>' + config.passCode + '</password>\n' +
        '</server>';

    // FIXME: Probably should sort out dynamic width of bits based on encoding strategy

    Ext.apply(self, {
      cls: 'nx-usertoken-UserTokenWindow',
      title: Nexus.usertoken.TITLE,
      border: false,
      closable: true,
      modal: true,
      autoHeight: true,
      width: 550,
      resizable: false,
      items: [
        {
          xtype: 'panel',
          items: [
            {
              xtype: 'component',
              cls: 'nx-usertoken-icon',
              html: icons.get('key').variant('x32').img
            },
            {
              xtype: 'label',
              html: 'User tokens are a combination of a name and password codes.<br/>' +
                  '<span style="font-weight: bold;">Keep these codes secret!</span><br/><br clear="both"/>' +
                  'Your user token is:<br/><br/>'
            },
            {
              xtype: 'textfield',
              name: 'nameCode',
              width: 100,
              readOnly: true,
              style: {
                padding: '2px',
                'font-family': 'monospace'
              },
              listeners: {
                render: self.installClickToSelectHandler
              }
            },
            {
              xtype: 'label',
              html: ':',
              style: {
                padding: '2px',
                'font-family': 'monospace'
              }
            },
            {
              xtype: 'textfield',
              name: 'passCode',
              width: 360,
              readOnly: true,
              style: {
                padding: '2px',
                'font-family': 'monospace'
              },
              listeners: {
                render: self.installClickToSelectHandler
              }
            },
            {
              xtype: 'label',
              html: '<br/><br/>Use the following in your Maven settings.xml:<br/><br/>'
            },
            {
              xtype: 'textarea',
              name: 'exampleXml',
              height: 90,
              width: 510,
              readOnly: true,
              style: {
                padding: '2px',
                'font-family': 'monospace'
              },
              listeners: {
                render: self.installClickToSelectHandler
              }
            },
            {
              xtype: 'label',
              html: '<br/><br/>This window will automatically close after one minute.'
            }
          ],
          buttons: [
            {
              text: 'Close',
              scope: self,
              handler: function () {
                self.close();
              }
            }
          ]
        }
      ],
      listeners: {
        show: function (component) {
          component.find('name', 'nameCode')[0].setValue(config.nameCode);
          component.find('name', 'passCode')[0].setValue(config.passCode);
          component.find('name', 'exampleXml')[0].setValue(exampleXml);

          // Automatically close the window
          (function () {
            if (self.isVisible()) { // ignore if already closed
              Nexus.usertoken.showMessage('Automatically closing window due to timeout');
              self.close();
            }
          }).defer(1 * 60 * 1000); // 1 minute
        }
      }
    });

    // Call super constructor
    self.constructor.superclass.constructor.apply(self, arguments);
  },

  /**
   * Installs click handler to automatically select component text.
   */
  installClickToSelectHandler: function (component) {
    component.getEl().on('click', function () {
      component.selectText();
    });
  }
});
