/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext,Sonatype,Nexus*/
define('Sonatype/repoServer/SmartProxyPanel', function() {

  Sonatype.Events.addListener('repositoryViewInit', function (cardPanel, rec) {
      var sp = Sonatype.lib.Permissions;
      if (sp.checkPermission('nexus:smartproxy:pub-sub', sp.READ) && rec.data.id /* not a new repo*/ ) {
          cardPanel.add(new Sonatype.repoServer.SmartProxyPanel({
              name: 'smartproxy',
              tabTitle: 'Smart Proxy',
              repoType: rec.data.repoType,
              payload: {
                  id: rec.data.id,
                  data: {
                      publish: false,
                      subscribe: false,
                      preemptiveFetch: false,
                      repositoryId: rec.data.id,
                      resourceURI: Sonatype.config.servicePath + '/smartproxy/pub-sub/' + rec.data.id
                  }
              }
          }));
      }
  });

  Sonatype.repoServer.SmartProxyPanel = function (config) {
      config = config || {};

      this.referenceData = {
          publish: false,
          subscribe: false,
          preemptiveFetch: false
      };

      var defaultConfig = {
          dataModifiers: {
              load: {},
              submit: {}
          }
      };

      Ext.apply(this, config, defaultConfig);

      this.downloadUpdatedArtifactsStore = new Ext.data.SimpleStore({
          fields: ['value', 'display'],
          data: [
              [false, 'Upon Request'],
              [true, 'Immediately']
          ]
      });

      Sonatype.repoServer.SmartProxyPanel.superclass.constructor.call(this, {
          region: 'center',
          trackResetOnLoad: true,
          autoScroll: true,
          border: false,
          frame: true,
          collapsible: false,
          collapsed: false,
          layoutConfig: {
              labelSeparator: ''
          },
          labelWidth: 225,
          width: 900,
          items: [
              {
                  xtype: 'fieldset',
                  checkboxToggle: false,
                  title: 'Publish',
                  anchor: Sonatype.view.FIELDSET_OFFSET,
                  collapsible: false,
                  autoHeight: true,
                  layoutConfig: {
                      labelSeparator: ''
                  },
                  items: [
                      {
                          xtype: 'checkbox',
                          fieldLabel: 'Publish Updates',
                          checked: false,
                          name: 'publish',
                          helpText: 'Inform downstream Nexus instances when items have been created or updated.'
                      },
                      {
                          xtype: 'fieldset',
                          itemId: 'sp_publish_status_fieldset',
                          checkboxToggle: false,
                          title: 'Status',
                          anchor: Sonatype.view.FIELDSET_OFFSET,
                          collapsible: false,
                          autoHeight: true,
                          layoutConfig: {
                              labelSeparator: ''
                          },
                          items: [
                              {
                                  xtype: 'displayfield',
                                  fieldLabel: 'Status',
                                  hideLabel: true,
                                  name: 'publishStatus',
                                  anchor: Sonatype.view.FIELDSET_OFFSET,
                                  width: 700
                              }
                          ]
                      }
                  ]
              },
              {
                  xtype: 'fieldset',
                  checkboxToggle: false,
                  title: 'Subscribe',
                  anchor: Sonatype.view.FIELDSET_OFFSET,
                  collapsible: false,
                  autoHeight: true,
                  /* subscription only for proxy repos */
                  hidden: config.repoType !== 'proxy',
                  layoutConfig: {
                      labelSeparator: ''
                  },
                  items: [
                      {
                          xtype: 'checkbox',
                          fieldLabel: 'Receive Updates',
                          checked: false,
                          name: 'subscribe',
                          helpText: 'Clear the local cache associated with remote items when they are created or updated.'
                      },
                      {
                          xtype: 'combo',
                          lazyInit: false,
                          fieldLabel: 'Download Updated Artifacts',
                          itemCls: 'required-field',
                          helpText: 'Download policy for update events: Upon Request - when requested from local Nexus, Immediately - as soon as notification is received.',
                          name: 'preemptiveFetch',
                          store: this.downloadUpdatedArtifactsStore,
                          valueField: 'value',
                          displayField: 'display',
                          editable: false,
                          forceSelection: true,
                          mode: 'local',
                          triggerAction: 'all',
                          selectOnFocus: true,
                          emptyText: 'Select...',
                          allowBlank: false,
                          value: 'pom',
                          minListWidth: 100
                      },
                      {
                          xtype: 'displayfield',
                          fieldLabel: 'Remote Fingerprint',
                          name: 'certificate.fingerprint',
                          style: 'text-decoration: underline; color: blue; cursor: pointer;',
                          listeners: {
                              render: {
                                  fn: function (component) {
                                      component.getEl().on('click', this.showCertificate.createDelegate(this));
                                  },
                                  scope: this
                              }
                          }
                      },
                      {
                          xtype: 'fieldset',
                          itemId: 'sp_subscribe_status_fieldset',
                          checkboxToggle: false,
                          title: 'Status',
                          anchor: Sonatype.view.FIELDSET_OFFSET,
                          collapsible: false,
                          autoHeight: true,
                          layoutConfig: {
                              labelSeparator: ''
                          },
                          items: [
                              {
                                  xtype: 'displayfield',
                                  fieldLabel: 'Status',
                                  hideLabel: true,
                                  name: 'subscriptionStatus',
                                  anchor: Sonatype.view.FIELDSET_OFFSET,
                                  width: 700
                              }
                          ],
                          listeners: {
                              show: {
                                  fn: function (component) {
                                      var cmp = Ext.get('sp_remote_cert_' + this.payload.id);
                                      if (cmp !== null ) {
                                          cmp.on('click', this.showCertificate.createDelegate(this));
                                      }
                                  },
                                  scope: this
                              }
                          }
                      },
                      { // TODO should use a store to avoid defining these hidden fields
                          xtype: 'textarea',
                          name: 'certificate.pem',
                          hideLabel: true,
                          hidden: true
                      },
                      {
                          xtype: 'textarea',
                          name: 'certificate.detail',
                          hideLabel: true,
                          hidden: true
                      }
                  ]
              }
          ]
      });

      this.on('actioncomplete', function (form, action) {
          if (action.type === 'sonatypeSubmit') {
              // reload form to get status info
              this.loadData();
          }

          var publishEnabledField, publishEnabled, publishStatusFieldSet, publishStatusField,
              subscriptionEnabledField, subscriptionEnabled, subscribeStatusFieldSet, subscriptionStatusField, fpField;

          publishEnabledField = form.findField('publish');
          publishEnabled = publishEnabledField && publishEnabledField.getValue();
          publishStatusFieldSet = this.find('itemId', 'sp_publish_status_fieldset')[0];
          publishStatusField = form.findField('publishStatus');
          if (typeof publishStatusFieldSet === 'object') {
              publishStatusFieldSet.setVisible(publishEnabled && publishStatusField.getValue());
          }

          subscriptionEnabledField = form.findField('subscribe');
          subscriptionEnabled = subscriptionEnabledField && subscriptionEnabledField.getValue();
          subscribeStatusFieldSet = this.find('itemId', 'sp_subscribe_status_fieldset')[0];
          subscriptionStatusField = form.findField('subscriptionStatus');
          if (typeof subscribeStatusFieldSet === 'object') {
              subscribeStatusFieldSet.setVisible(subscriptionEnabled && subscriptionStatusField.getValue());
          }

          fpField = form.findField('certificate.fingerprint');
          if (fpField !== null) {
              fpField.setVisible(subscriptionEnabled && fpField.getValue());
          }
      }.createDelegate(this));
  };

  Ext.extend(Sonatype.repoServer.SmartProxyPanel, Sonatype.ext.FormPanel, Ext.apply(Sonatype.repoServer.SmartProxyCertificateUtils, {
      showWindow: function (record) {
          var window = this.showAddCertificateWindow('Add Trusted Certificate', record);
          window.find('name', 'certificate')[0].disable();

          setTimeout(function () {
              var description = window.find('name', 'description')[0];
              if (description) {
                  description.focus();
                  description.clearInvalid();
              }
          }, 200);
      },
      showCertificate: function (element) {
          var fingerprint = this.getForm().findField('certificate.fingerprint').getValue();

          Ext.Ajax.request({
              url: Sonatype.config.servicePath + '/smartproxy/trusted-keys',
              method: 'PUT',
              jsonData: {
                  data: {
                      // FIXME use real response object once this request has a different URL
                      id: fingerprint
                  }
              },
              success: function (resp) {
                  var record = new Ext.data.Record(Ext.decode(resp.responseText).data);
                  this.showWindow(record);
              },
              failure: function (resp) {
                  var pem, detail, record, msg, cmp;

                  if (resp.status === 404) {
                      Sonatype.MessageBox.hide();

                      pem = this.getForm().findField('certificate.pem').getValue();
                      detail = this.getForm().findField('certificate.detail').getValue();
                      record = new Ext.data.Record({
                          certificate: {
                              pem: pem,
                              detail: detail
                          }
                      });
                      this.showWindow(record);
                  }
                  else if (resp.status === 400) {
                      Sonatype.MessageBox.hide();

                      msg = Ext.decode(resp.responseText).validationErrors[0].message;
                      Sonatype.MessageBox.show({
                          title: 'Trust Capability Error',
                          msg: msg,
                          buttons: Sonatype.MessageBox.OK,
                          icon: Sonatype.MessageBox.ERROR
                      });

                      cmp = Ext.get('sp_goto_capabilities');
                      if (cmp !== null ) {
                          cmp.on('click',
                              function (element) {
                                  Sonatype.MessageBox.hide();
                              }
                          );
                      }
                  }
              },
              scope: this
          });
      }
  }));
});


